"""Status model."""
from chicken_dinner.constants import STATUS_URL


class Status(object):
    """Status model.

    Contains information about the status of the PUBG API.

    :param pubg: an instance of the :class:`chicken_dinner.pubgapi.PUBG` class
    """

    def __init__(self, pubg):
        self._pubg = pubg
        #: The API response for this object
        self.response = self._pubg._core.status()

    @property
    def data(self):
        """The status data payload."""
        return self.response["data"]

    @property
    def id(self):
        """Get the id of the status."""
        return self.data["id"]

    @property
    def version(self):
        """Get the API version."""
        return self.data["attributes"]["version"]

    @property
    def released_at(self):
        """Get the timestamp this version was released."""
        return self.data["attributes"]["releasedAt"]

    @property
    def url(self):
        """The status endpoint URL."""
        return STATUS_URL

    def refresh(self):
        """Refresh the api status."""
        self.response = self._pubg._core.status()



"""Version information."""
__title__ = "chicken_dinner"
__description__ = "PUBG JSON API wrapper and playback visualizer."
__url__ = "https://github.com/crflynn/chicken-dinner"
__version__ = "0.2.4"
__author__ = "Christopher Flynn"
__author_email__ = "crf204@gmail.com"
__license__ = "MIT"
__copyright__ = "Copyright 2018 Christopher Flynn"
__docs_copyright__ = "2018 Christopher Flynn"



"""Players filtered results model."""
from .player import Player


class Players(object):
    """Players model.

    An iterable containing multiple instances of the class
    :class:`chicken_dinner.models.Player`.

    :param pubg: an instance of the class :class:`chicken_dinner.pubgapi.PUBG`
    :param str filter_type: either "player_ids" or "player_names"
    :param list filter_value: a list of ``player_ids`` or ``player_names``
        corresponding to the ``filter_type`` parameter
    :param str shard: the shard for the seasons response
    """

    def __init__(self, pubg, filter_type, filter_value, shard=None):
        self._pubg = pubg
        self._shard = shard
        #: The filter type for this Players query
        self.filter_type = filter_type
        #: The filter value for this Players query
        self.filter_value = filter_value
        #: The API response for this object
        self.response = self._pubg._core.players(
            filter_type, filter_value, shard
        )
        self._players = [
            Player.from_data(pubg, p, shard=shard) for p in self.data
        ]

    def __getitem__(self, idx):
        return self._players[idx]

    @property
    def shard(self):
        """The shard for this player."""
        return self._shard or self._pubg.shard

    @property
    def data(self):
        """The data payload of the repsonse."""
        return self.response["data"]

    @property
    def ids(self):
        """A list of player ids corresponding to the players."""
        return [p.id for p in self._players]

    @property
    def names(self):
        """A list of player names corresponding to the players."""
        return [p.name for p in self._players]

    @property
    def url(self):
        """The URL for this players resource."""
        return self.response["links"]["self"]

    def ids_to_names(self):
        """Construct a mapping of ``player_id`` to ``player_name``."""
        return {p.id: p.name for p in self._players}

    def names_to_ids(self):
        """Construct a mapping of ``player_name`` to ``player_id``."""
        return {p.name: p.id for p in self._players}

    def shared_matches(self):
        """Return a list of matches in which all players participated.

        :return: list of ``match_ids``
        """
        return list(set.intersection(*[set(p.match_ids) for p in self._players]))



from collections import MutableMapping


class CaseInsensitiveDict(MutableMapping):

    def __init__(self, data=None, **kwargs):
        self._store = {}
        if data is None:
            data = {}
        self.update(data, **kwargs)

    def __delitem__(self, key):
        del self._store[key.replace("_", "").lower()]

    def __getitem__(self, key):
        return self._store[key.replace("_", "").lower()][1]

    def __setitem__(self, key, value):
        self._store[key.replace("_", "").lower()] = (key, value)

    def __iter__(self):
        return (cased_key for cased_key, mapped_value in self._store.values())

    def __len__(self):
        return len(self._store)

    def __eq__(self, other):
        if isinstance(other, collections.Mapping):
            other = CaseInsensitiveDict(other)
        else:
            return NotImplemented
        return dict(self.lower_items()) == dict(other.lower_items())

    def __repr__(self):
        return str(dict(self.items()))

    def lower_items(self):
        return (
            (lowerkey, keyval[1])
            for (lowerkey, keyval)
            in self._store.items()
        )

    def copy(self):
        return CaseInsensitiveDict(self._store.values())

    @classmethod
    def from_json(cls, json):
        data = {}
        if isinstance(json, list):
            return [cls.from_json(element) for element in json]
        elif isinstance(json, dict):
            for key, value in json.items():
                if isinstance(value, (list, dict)):
                    data[key] = cls.from_json(value)
                else:
                    data[key] = value
        else:
            return json
        return cls(data)



"""Player-season stats model."""
import re

from chicken_dinner.constants import GROUPS
from chicken_dinner.constants import PERSPECTIVES
from chicken_dinner.constants import game_mode_to_gp
from chicken_dinner.constants import gp_to_game_mode
from chicken_dinner.constants import gp_to_matches
from chicken_dinner.constants import matches_to_gp
from chicken_dinner.util import camel_to_snake


class PlayerSeason(object):
    """Player-season model.

    An object containing data about a player's season data and stats.

    :param pubg: an instance of the class :class:`chicken_dinner.pubgapi.PUBG`
    :param str player_id: the player's account id
    :param str season_id: a season id for the player data
    :param str shard: the shard for the seasons response
    """

    def __init__(self, pubg, player_id, season_id, shard=None):
        self._pubg = pubg
        self._shard = shard
        self._player_id = player_id
        self._season_id = season_id
        #: The API response for this object.
        self.response = self._pubg._core.player_season(
            player_id, season_id, shard
        )

    @property
    def shard(self):
        """The shard for this player-season."""
        return self._shard or self._pubg.shard

    @property
    def id(self):
        """The ids of the player and season.

        :return: a dict with keys ``player_id`` and ``season_id``
        """
        return {
            "player_id": self._player_id,
            "season_id": self._season_id,
        }

    @property
    def data(self):
        """The data payload from the response."""
        return self.response["data"]

    @property
    def player_id(self):
        """The player's account id."""
        return self._player_id

    @property
    def season_id(self):
        """The season id."""
        return self._season_id

    @property
    def url(self):
        """A URL for this player-season resource."""
        return self.response["links"]["self"]

    def game_mode_stats(self, group=None, perspective=None):
        """Get game mode states for a group-perspective (game mode).

        :param str group: (optional) either ``solo``, ``duo``, or ``squad``.
            If not specified returns stats for all group modes
        :param str perspective: (optional) either ``tpp`` or ``fpp``. If not
            specified returns stats for all perspectives
        """
        data = self.data["attributes"]["gameModeStats"]
        new_stats = {}
        if group is None and perspective is None:
            for mode, stats in data.items():
                new_stats[game_mode_to_gp[mode]] = {
                    camel_to_snake(stat): value for stat, value in stats.items()
                }
        elif group is None and perspective is not None:
            for mode, stats in data.items():
                if perspective in mode:
                    new_stats[game_mode_to_gp[mode]] = {
                        camel_to_snake(stat): value for stat, value in stats.items()
                    }
        elif group is not None and perspective is None:
            for mode, stats in data.items():
                if group in mode:
                    new_stats[game_mode_to_gp[mode]] = {
                        camel_to_snake(stat): value for stat, value in stats.items()
                    }
        else:
            group_perspective = group + "-" + perspective
            mode = gp_to_game_mode[group_perspective]
            stats = data[mode]
            new_stats = {
                camel_to_snake(stat): value for stat, value in stats.items()
            }
        return new_stats

    def match_ids(self, group=None, perspective=None, flat=False):
        """Get recent match_ids for a group-perspective (game mode).

        :param str group: (optional) either ``solo``, ``duo``, or ``squad``.
            If not specified returns ``match_ids`` for all group modes
        :param str perspective: (optional) either ``tpp`` or ``fpp``. If not
            specified returns ``match_ids`` for all perspectives
        :param bool flat: if ``True`` returns match ids in a list
        """
        data = self.data["relationships"]
        match_ids = {}
        if group is None and perspective is None:
            for mode, matches in data.items():
                if "matches" in mode:
                    match_ids[matches_to_gp[mode]] = [
                        match["id"] for match in matches["data"]
                    ]
        elif group is None and perspective is not None:
            for mode, matches in data.items():
                if perspective in mode.lower():
                    match_ids[matches_to_gp[mode]] = [
                        match["id"] for match in matches["data"]
                    ]
        elif group is not None and perspective is None:
            for mode, matches in data.items():
                if group in mode.lower():
                    match_ids[matches_to_gp[mode]] = [
                        match["id"] for match in matches["data"]
                    ]
        else:
            group_perspective = group + "_" + perspective
            mode = gp_to_matches[group_perspective]
            matches = data[mode]
            match_ids = [match["id"] for match in matches["data"]]
        if flat and isinstance(match_ids, dict):
            flat_ids = []
            for mode, matches in match_ids.items():
                flat_ids = flat_ids + matches
            return flat_ids
        return match_ids



"""Match roster model."""
from .participant import Participant
from chicken_dinner.util import camel_to_snake
from chicken_dinner.constants import true_false


class Roster(object):
    """Roster model.

    :param pubg: a PUBG instance
    :param match: the match object associated with this roster
    :param data: the data payload associated with this roster response object
    :param str shard: the shard for the match associated with this roster
    """

    def __init__(self, pubg, match, data, shard=None):
        self._pubg = pubg
        self._shard = shard
        #: The Match instance for this roster
        self.match = match
        #: The data payload associated with this Roster
        self.data = data
        #: A list of Participant instances for this roster
        self.participants = [
            Participant(pubg, match, self, match._participant_data[participant["id"]], shard)
            for participant in self.data["relationships"]["participants"]["data"]
        ]
        #: Stats for this roster
        self.stats = {
            camel_to_snake(k): v
            for k, v in self.data["attributes"]["stats"].items()
        }
        self.stats["won"] = self.data["attributes"]["won"]

    @property
    def shard(self):
        """The shard for the match associated with this roster."""
        return self._shard or self._pubg.shard

    @property
    def id(self):
        """The roster id."""
        return self.data["id"]

    @property
    def participant_ids(self):
        """A list of match-specific participant ids for this roster."""
        return [p.id for p in self.participants]

    @property
    def player_ids(self):
        """A list of player account ids for this roster."""
        return [p.player_id for p in self.participants]

    @property
    def player_names(self):
        """A list of player names for this roster."""
        return [p.name for p in self.participants]

    @property
    def won(self):
        """Whether or not this roster won the associated match."""
        return true_false[self.stats["won"]]



"""Individual player model."""
# flake8: noqa
from .season import Season


class Player(object):
    """Player model.

    An object encapsulating metadata about a PUBG player.

    :param pubg: an instance of the class :class:`chicken_dinner.pubgapi.PUBG`
    :param str player_id: the player's account id
    :param dict data: (optional) the data payload from the player response
    :param str shard: the shard for the seasons response
    """

    def __init__(self, pubg, player_id, data=None, shard=None):
        self._pubg = pubg
        self._shard = shard
        if data is None:
            #: The API response for this object.
            self.response = self._pubg._core.player(player_id, shard)
        else:
            self.response = {"data": data}

    @property
    def shard(self):
        """The shard for this player."""
        return self._shard or self._pubg.shard

    # # deprecated
    # @property
    # def created_at(self):
    #     """When the player was created."""
    #     return self.data["attributes"]["createdAt"]

    @property
    def data(self):
        """The data payload of the player response."""
        return self.response["data"]

    @property
    def id(self):
        """The player account id."""
        return self.data["id"]

    @property
    def match_ids(self):
        """A list of the player's most recent match ids."""
        return [match["id"] for match in self.data["relationships"]["matches"]["data"]]

    @property
    def name(self):
        """The player's in-game name."""
        return self.data["attributes"]["name"]

    @property
    def patch_version(self):
        """The last patch version on which the player has played."""
        return self.data["attributes"]["patchVersion"]

    @property
    def stats(self):
        """Stats for the player."""
        return self.data["attributes"]["stats"]

    @property
    def title_id(self):
        """The title_id for the data."""
        return self.data["attributes"]["titleId"]

    # # deprecated
    # @property
    # def updated_at(self):
    #     """When this data was last updated."""
    #     return self.data["attributes"]["updatedAt"]

    @property
    def url(self):
        """The URL for this player resource."""
        try:
            return self.data["links"]["self"]
        except IndexError as exc:
            return SHARD_URL + self.shard + "/players/" + str(self.id)

    @classmethod
    def from_data(cls, pubg, data, shard=None):
        """Constructor for a player object with a data payload.

        :param pubg: an instance of the class :class:`chicken_dinner.pubgapi.PUBG`
        :param dict data: the data payload from the player response
        :param str shard: the shard for the seasons response
        """
        return cls(pubg, data["id"], data=data, shard=shard)

    def get_season(self, season_id):
        """Get a season response for a specific season.

        :param season_id: a ``season_id`` or
            :class:`chicken_dinner.models.Season` object from which to get
            player-season data. Use the string "current" to get the
            current season.
        :return: a :class:`chicken_dinner.models.PlayerSeason` instance
        """
        if isinstance(season_id, Season):
            season_id = season_id.id
        return self._pubg.player_season(self.id, season_id, self.shard)

    def get_current_season(self):
        """Get the player-season data for the current season.

        :return: a :class:`chicken_dinner.models.PlayerSeason` instance for
            the current season
        """
        return self.get_season("current")



from .player_season import PlayerSeason
from .player import Player
from .players import Players
from .samples import Samples
from .season import Season
from .seasons import Seasons
from .status import Status
